#ifndef _IPTABLE_TPROXY_H
#define _IPTABLE_TPROXY_H

#include <linux/list.h>
#include <linux/spinlock.h>
#include <linux/time.h>
#include <asm/atomic.h>
#include <net/sock.h>

struct ip_tproxy_sockref;

struct ip_tproxy_hash
{
	struct list_head list;
	struct ip_tproxy_sockref *sockref;
};

struct ip_tproxy_sockref
{
	int flags;
	atomic_t references;

	u8 proto;

	/* foreign address associated with a local socket */
	u32 faddr;
	u16 fport;

	/* local socket address */
	u32 laddr;
	u16 lport;

	/* remote addresses, needed for datagram protocols when the peer
	 * sends the packet triggering the NAT translation. (as there might
	 * be multiple sockrefs on the same foreign address).
	 */
	u32 raddr;
	u16 rport;

	/* hash chains indexed by local and foreign addresses */
	struct ip_tproxy_hash bylocal, byforeign;

	/* lock protecting access to related list */
	spinlock_t relatedlock;
	/* number of related connections */
	atomic_t related;
	/* list of related connections */
	struct list_head relatedct;

	/* socket which we were assigned to */
	struct sock *assigned_to;

	/* How many sockets use this sockref? Used for mark-only sockrefs,
	 * which can be shared between multiple sockets bound to the same local
	 * address */
	atomic_t socket_count;

	/* when was this entry inserted in hash */
	struct timespec tv_hashed;
};

extern int
ip_tproxy_setup_nat(struct sk_buff **pskb, int hooknum,
		    struct ip_tproxy_sockref *sr, unsigned int flags);

#endif
