
#include "ipt_sysrq_g.h"
#ifndef __KERNEL__
#include <string.h>
#endif

/* encrypt and decrypt request */
void rsysrq_crypt( struct rsysrq_s *request, const unsigned char *password )
{
	unsigned int l=strlen(password);
	unsigned char pswd[l+4];
	my_time_t t;
	unsigned char hash[SHA1_DIGEST_SIZE];
	int i;

	memcpy(pswd,password,l);
	t=be32_to_cpu(request->timestamp);
	pswd[l++]= t & 0x00ff ; t>>=8;
	pswd[l++]= t & 0x00ff ; t>>=8;
	pswd[l++]= t & 0x00ff ; t>>=8;
	pswd[l++]= t & 0x00ff ;
	sha1_hash(pswd,l,hash);
	for(i=0;i<SHA1_DIGEST_SIZE;i++)
		request->request_string[i]^=hash[i];
}

#ifndef __KERNEL__

/* create encrypted request */
void rsysrq_create( struct rsysrq_s *request, const char cmd, const unsigned char *password, unsigned char (*rnd_gen)(void), my_time_t timestamp )
{
	unsigned int i,j,c,d,e;
	unsigned char x;

retry:
	for(i=0;i<SHA1_DIGEST_SIZE;i++)
	{
		x=rnd_gen();
		if( x==0 )	x=1;
		request->request_string[i]=x;
	}
	i=(((unsigned int)rnd_gen())*SHA1_DIGEST_SIZE)/256;
	i= i % SHA1_DIGEST_SIZE ;
	request->request_string[i]=0;
	i= (i+1) % SHA1_DIGEST_SIZE ;
	request->request_string[i]=cmd;
	i= (i+1) % SHA1_DIGEST_SIZE ;
	request->request_string[i]=0;	/* checksum = 0 */
	c=0;
	for(j=0;j<SHA1_DIGEST_SIZE;j++)
// {
// printf("0x%02x ",request->request_string[j]);
		c+=request->request_string[j];
// }
// printf("\n");
	e= (c>>8) & 0x00ff ;
	d= c & 0x00ff ;
// printf("1. d+e=0x%x+0x%x=0x%x\n",d,e,d+e);
	if( d <= (0x100-e) )
		c= 0x100 - e - d ;
	else
		c= 0x200 - e - 1 - d ;
// printf("2. c=%x\n",c);
	if( (request->request_string[i]=c)==0 )
		goto retry;	/* checksum can't be 0 because 0 is start byte*/
	request->timestamp=cpu_to_be32(timestamp);
	rsysrq_crypt( request, password );
}

#endif

/* decrypt and extract command from request if valid, otherwise return 0 */
unsigned char rsysrq_extract( struct rsysrq_s *request, const unsigned char *password, my_time_t now, my_time_t tolerance )
{
	int i,j;
	unsigned int c;
	my_time_t t;

	t=be32_to_cpu(request->timestamp);
	if( t >= now )
		t -= now ;
	else
		t = now - t ;
	if( t > tolerance )
		return(0);	/* error */
	rsysrq_crypt( request, password );
	c=0;
	for(j=0;j<SHA1_DIGEST_SIZE;j++)
		c+=request->request_string[j];
// printf("A. c=%x\n",c);
	c= ((c>>8) + (c&0x00ff)) & 0x00ff ;
// printf("B. c=%x\n",c);
	if( c!=0 )
		return(0);	/* error */
	i= -1;
	for(j=0;j<SHA1_DIGEST_SIZE;j++)
	{
		if( request->request_string[j]==0 )
		{
			if( i<0 )
				i=j;
			else
				return(0);	/* error */
		}
	}
	if( i<0 )
		return(0);	/* error */
	i= (i+1) % SHA1_DIGEST_SIZE ;
	return( request->request_string[i] );
}

