/*
	Copyright © Jan Engelhardt <jengelh [at] gmx.de>, 2006 - 2007

	This file is part of quad_dsp. quad_dsp is free software; you
	can redistribute it and/or modify it under the terms of the
	GNU General Public License as published by the Free Software
	Foundation; however ONLY version 2 of the License. For
	details, see the file named "LICENSE.GPL2".
*/
#include <linux/compiler.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/soundcard.h>
#include <linux/types.h>
#include <asm/uaccess.h>
#include "quad_dsp.h"

static ssize_t fx_dsr(struct file *filp, const char __user *buf, size_t count,
    loff_t *ppos)
{
	struct prv *p = filp->private_data;
	int eax = 0, ebx = 0;
	mm_segment_t oldfs;
	loff_t pos = 0;
	unsigned int max = min(count, REMIX_BSIZE),
	             front_idx = 0, rear_idx = 0;

	if(count > max && !p->warning) {
		printk(KERN_WARNING PREFIX "%s: pid %d wants to write %u "
		       "bytes, but there are only %u free\n", __FUNCTION__,
		       current->pid, count, max);
		++p->warning;
	}

	if(copy_from_user(p->buf_front, buf, max) != 0)
		return -EFAULT;

	if(p->fmt == AFMT_S16_LE) {
		signed short *pf = p->buf_front, *pr = p->buf_rear;
		long left, right, sub;
		max /= 2;
		while(front_idx < max) {
			left           = pf[front_idx++];
			right          = pf[front_idx++];
			sub            = pv_squash16(left - right);
			pr[rear_idx++] = sub;
			pr[rear_idx++] = sub;
		}
		front_idx *= 2;
		rear_idx  *= 2;
	} else if(p->fmt == AFMT_S8) {
		signed char *pf = p->buf_front, *pr = p->buf_rear;
		long left, right, sub;
		while(front_idx < max) {
			left           = pf[front_idx++];
			right          = pf[front_idx++];
			sub            = pv_squash16(left - right);
			pr[rear_idx++] = sub;
			pr[rear_idx++] = sub;
		}
	} else if(p->fmt == AFMT_U8) {
		unsigned char *pf = p->buf_front, *pr = p->buf_rear;
		int left, right, sub;
		while(front_idx < max) {
			left           = pf[front_idx++] - 128;
			right          = pf[front_idx++] - 128;
			sub            = pv_squash16(left - right);
			pr[rear_idx++] = sub + 128;
			pr[rear_idx++] = sub + 128;
		}
	} else {
		return -EINVAL;
	}

	oldfs = get_fs();
	set_fs(KERNEL_DS);

	if((eax = vfs_write(p->front, p->buf_front, front_idx, &pos)) < 0) {
		set_fs(oldfs);
		return eax;
	}
	pos = 0;
	if((ebx = vfs_write(p->rear, p->buf_rear, rear_idx, &pos)) < 0) {
		set_fs(oldfs);
		return ebx;
	}

	set_fs(oldfs);
	return (eax > ebx) ? eax : ebx;
}

struct filter Qfilter_dsr = {
	.write_mono = fx_dup,
	.write_dual = fx_dsr,
	.write_plus = fx_split,
	.misc_dev   = {.name = "Qdsp_dsr"},
};
